library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity eppcu is
    Port (
	--the eppcu's datalines that run to the parallel port
	epp_data					: inout	std_logic_vector(7 downto 0);
	epp_write 				: in 	std_logic;
	epp_wait 				: out std_logic;
	epp_data_strobe 		: in 	std_logic;
	epp_address_strobe 	: in 	std_logic;
	epp_irq					: out	std_logic; --not used but set to zero
	--           epp_reset 		: in 	std_logic;

	--the eppcu's locic interface to the rest of the tvc		 
	tvc_clock				: in	std_logic;
	tvc_reset				: in 	std_logic;

	--interface to the block_reader memory user (mu)
	br_fb_address 			: out std_logic_vector(31 downto 0);
	br_fb_data				: in std_logic_vector(31 downto 0);
	br_start_strobe		: out std_logic;
	br_idle					: in std_logic;

	--interface to the block_writer memory user (mu)
	bw_fb_address 			: out std_logic_vector(31 downto 0);
	bw_fb_data				: out std_logic_vector(31 downto 0);
	bw_start_strobe		: out std_logic;
	bw_idle					: in std_logic;

	--interface to the block_set memory user
	bs_fb_address 			: out std_logic_vector(31 downto 2);
	bs_num_bytes			: out std_logic_vector(23 downto 2);
	bs_fb_value				: out std_logic_vector(31 downto 0);
	bs_start_strobe		: out std_logic;
	bs_idle					: in std_logic;

	--interface to the polyscanline mu
	ps_fb_address 			: out std_logic_vector(31 downto 0);
	ps_db_address 			: out std_logic_vector(31 downto 0);
	ps_start_z_depth		: out std_logic_vector(31 downto 0);
	ps_z_increment			: out std_logic_vector(31 downto 0);
	ps_scanline_length	: out std_logic_vector(15 downto 0);
	ps_pix_value			: out std_logic_vector(7 downto 0);
	ps_start_strobe		: out std_logic;
	ps_idle					: in std_logic

	--eppcu_address_0	: out	std_logic_vector(7 downto 0);
	--eppcu_address_1	: out	std_logic_vector(7 downto 0);
	--eppcu_adderss_2	: out	std_logic_vector(7 downto 0);
	--eppcu_pix_value	: inout	std_logic_vector(7 downto 0)
	);	 
end eppcu;

architecture Behavioral of eppcu is
	--the epp host (computer) writes an address, then next writes a data byte to go into the desired address's register
	signal epp_address : std_logic_vector(7 downto 0);

	signal latched_epp_write 				: std_logic;
	signal latched_epp_data_strobe 		: std_logic;
	signal latched_epp_address_strobe 	: std_logic;
	signal epp_error : std_logic;
	signal waiting_on_br : std_logic; --when the pr goes idle put its pr_fb_data into fb_pix registers

	--registers that hold data for read/write from the epp port
	signal epp_dr_0 : std_logic_vector(7 downto 0);
	signal epp_dr_1 : std_logic_vector(7 downto 0);
	signal epp_dr_2 : std_logic_vector(7 downto 0);
	signal epp_dr_3 : std_logic_vector(7 downto 0);
	signal epp_dr_4 : std_logic_vector(7 downto 0);
	signal epp_dr_5 : std_logic_vector(7 downto 0);
	signal epp_dr_6 : std_logic_vector(7 downto 0);
	signal epp_dr_7 : std_logic_vector(7 downto 0);
	signal epp_dr_8 : std_logic_vector(7 downto 0);
	signal epp_dr_9 : std_logic_vector(7 downto 0);
	signal epp_dr_10 : std_logic_vector(7 downto 0);
	signal epp_dr_11 : std_logic_vector(7 downto 0);
	signal epp_dr_12 : std_logic_vector(7 downto 0);
	signal epp_dr_13 : std_logic_vector(7 downto 0);
	signal epp_dr_14 : std_logic_vector(7 downto 0);
	signal epp_dr_15 : std_logic_vector(7 downto 0);
	signal epp_dr_16 : std_logic_vector(7 downto 0);
	signal epp_dr_17 : std_logic_vector(7 downto 0);
	signal epp_dr_18 : std_logic_vector(7 downto 0);
	signal epp_dr_19 : std_logic_vector(7 downto 0);

	--the epp port is slow compared to the rest of the fpga, so we do a lot of waiting
	signal wait_timer : std_logic_vector(11 downto 0);
	type EPP_INT_STATE is ( EPP_NORMAL, EPP_DELAY_HOST_ADDRESS_WRITE_START, EPP_DELAY_HOST_DATA_WRITE_START, EPP_DELAY_NOTIFY_HOST_DATA_READ, EPP_ENDING_HOST_DATA_READ, EPP_ENDING_HOST_DATA_WRITE, EPP_ENDING_HOST_ADDRESS_WRITE,
									EPP_STARTING_BR, EPP_STARTING_BW, EPP_STARTING_BS, EPP_STARTING_PS, EPP_SIMPLE_DELAY);
	signal epp_state : EPP_INT_STATE;

	--when in state EPP_SIMPLE_DELAY we wait until wait_timer == simple_delay_time
	--then set our epp_state to be simple_delay_post_state
	signal simple_delay_post_state 			: EPP_INT_STATE;
 	signal simple_delay_time 					: std_logic_vector(11 downto 0);

	procedure BeginSimpleDelay(
		constant state_after_delay 	: in EPP_INT_STATE;
		constant des_delay_time 		: in std_logic_vector(11 downto 0);
		signal epp_state 						: out EPP_INT_STATE;
		signal simple_delay_post_state 	: out EPP_INT_STATE;
		signal simple_delay_time 			: out std_logic_vector(11 downto 0);
		variable reset_wait_timer			: out boolean ) is
	begin
		simple_delay_post_state <= state_after_delay;
		simple_delay_time <= des_delay_time;
		epp_state <= EPP_SIMPLE_DELAY;
		reset_wait_timer := true;
	end BeginSimpleDelay;

begin
	process(tvc_clock)
	variable ending_host_data_write : boolean;
	variable reset_wait_timer : boolean;
	begin
	if tvc_clock = '1' and tvc_clock'event then
	if tvc_reset = '1' then
		epp_error <= '0';
		waiting_on_br <= '0';
		epp_address <= X"00";
		--epp_dr_0 <= X"00";
		--epp_dr_1 <= X"00";
		--epp_dr_2 <= X"00";
		--epp_dr_3 <= X"00";
		--epp_dr_4 <= X"00";		
		--epp_dr_5 <= X"00";		
		--epp_dr_6 <= X"00";		
		--epp_dr_7 <= X"00";
		--epp_dr_8 <= X"00";
		--epp_dr_9 <= X"00";
		--epp_dr_10 <= X"00";
		--epp_dr_11 <= X"00";
		--epp_dr_12 <= X"00";
		--epp_dr_13 <= X"00";
		--epp_dr_14 <= X"00";
		--epp_dr_15 <= X"00";
		--epp_dr_16 <= X"00";
		--epp_dr_17 <= X"00";
		--epp_dr_18 <= X"00";
		--epp_dr_19 <= X"00";
		--br_fb_address 	<= X"0000_0000";
		epp_data <= (others => 'Z');
		epp_wait <= '0';
		br_start_strobe <= '0';

		--bw_fb_address 	<= X"0000_0000";
		--bw_fb_data <= X"0000_0000";
		bw_start_strobe <= '0';

		--bs_fb_address <= X"0000_000" & "00";
		--bs_num_bytes <= X"00_000" & "00";
		--bs_fb_value	<= X"0000_0000";
		bs_start_strobe <= '0';

		--ps_fb_address <= X"0000_0000";
		--ps_db_address <= X"0000_0000";
		--ps_start_z_depth <= X"0000_0000";
		--ps_z_increment <= X"0000_0000";
		--ps_scanline_length <= X"0000";
		--ps_pix_value <= X"FF";
		ps_start_strobe <= '0';
		
		epp_state <= EPP_NORMAL;
		--simple_delay_post_state <= EPP_NORMAL;
		
		wait_timer <= X"000";
		--simple_delay_time <= X"000";
		
		epp_irq <= '0';		
	else	--not a reset
		ending_host_data_write := false;
		reset_wait_timer := false;
		
		latched_epp_write <= epp_write;
      latched_epp_data_strobe <= epp_data_strobe; 
      latched_epp_address_strobe <= epp_address_strobe;
		
	case epp_state is
	 when EPP_ENDING_HOST_DATA_READ =>
		--before we enter this state we have already waited X"014" ticks, so we are immediatly ready
		--to check for latched_epp_data_strobe to be high
		if latched_epp_data_strobe = '1' then
			epp_data <= (others => 'Z');
			epp_state <= EPP_NORMAL;
			epp_wait <= '0';
		elsif wait_timer = X"1f4" then  --a ?hackish? timeout
			epp_data <= (others=> 'Z');
			epp_state <= EPP_NORMAL;
			epp_wait <= '0';
		end if;
	when EPP_ENDING_HOST_DATA_WRITE =>
		--before we enter this state we have already waited X"014" ticks, so we are immediatly ready
		--to check for latched_epp_data_strobe to be high
		if latched_epp_data_strobe = '1' then
			epp_wait <= '0';
			epp_state <= EPP_NORMAL;
			epp_data <= (others => 'Z');
		elsif wait_timer = X"1f4" then --timeout
			epp_wait <= '0';
			epp_state <= EPP_NORMAL;
			epp_data <= (others => 'Z');
		end if;
	when EPP_ENDING_HOST_ADDRESS_WRITE =>
		--before we enter this state we have already waited X"014" ticks, so we are immediatly ready
		--to check for latched_epp_data_strobe to be high
		if latched_epp_address_strobe = '1' then
			epp_wait <= '0';
			epp_state <= EPP_NORMAL;
			epp_data <= (others => 'Z');
		elsif wait_timer = X"1f4" then --timeout
			epp_wait <= '0';
			epp_state <= EPP_NORMAL;
			epp_data <= (others => 'Z');
		end if;
	when EPP_NORMAL =>
		if waiting_on_br = '1' and br_idle = '1' then
	    	epp_dr_4 <= br_fb_data(31 downto 24);
			epp_dr_5 <= br_fb_data(23 downto 16);
			epp_dr_6 <= br_fb_data(15 downto 8);
			epp_dr_7 <= br_fb_data(7 downto 0);
			waiting_on_br <= '0';
		end if;

		if latched_epp_address_strobe = '0' and latched_epp_write = '0' then
			BeginSimpleDelay(EPP_DELAY_HOST_ADDRESS_WRITE_START, X"010", epp_state, simple_delay_post_state, simple_delay_time, reset_wait_timer);
		elsif latched_epp_data_strobe = '0' and latched_epp_write = '0' then
			BeginSimpleDelay(EPP_DELAY_HOST_DATA_WRITE_START, X"010", epp_state, simple_delay_post_state, simple_delay_time, reset_wait_timer);
		elsif latched_epp_data_strobe = '0' and latched_epp_write = '1' then
		   case epp_address is
				when X"00" =>
					epp_data <= epp_dr_0;
				when X"01" =>
					epp_data <= epp_dr_1;
				when X"02" =>
					epp_data <= epp_dr_2;
				when X"03" =>
					epp_data <= epp_dr_3;
				when X"04" =>
					epp_data <= epp_dr_4;
				when X"05" =>
					epp_data <= epp_dr_5;
				when X"06" =>
					epp_data <= epp_dr_6;
				when X"07" =>
					epp_data <= epp_dr_7;
				when X"08" =>
					epp_data <= epp_dr_8;
				when X"09" =>
					epp_data <= epp_dr_9;
				when X"0a" =>
					epp_data <= epp_dr_10;
				when X"0b" =>
					epp_data <= epp_dr_11;
				when X"0c" =>
					epp_data <= epp_dr_12;
				when X"0d" =>
					epp_data <= epp_dr_13;
				when X"0e" =>
					epp_data <= epp_dr_14;
				when X"0f" =>
					epp_data <= epp_dr_15;
				when X"10" =>
					epp_data <= epp_dr_16;
				when X"11" =>
					epp_data <= epp_dr_17;
				when X"12" =>
					epp_data <= epp_dr_18;
				when X"13" =>
					epp_data <= epp_dr_19;
				when X"15" =>
					epp_data <= epp_error & br_idle & bw_idle & bs_idle & ps_idle & "000";
				when others =>
					epp_data <= X"00";
			end case;
			BeginSimpleDelay(EPP_DELAY_NOTIFY_HOST_DATA_READ, X"010", epp_state, simple_delay_post_state, simple_delay_time, reset_wait_timer);
		end if; --done read and write
	when EPP_DELAY_HOST_ADDRESS_WRITE_START =>
	  	epp_address <= epp_data;
		epp_wait <= '1';
		BeginSimpleDelay(EPP_ENDING_HOST_ADDRESS_WRITE, X"010", epp_state, simple_delay_post_state, simple_delay_time, reset_wait_timer);
	when	EPP_DELAY_HOST_DATA_WRITE_START =>
		ending_host_data_write := true;
		case epp_address is
		when X"00" =>
			epp_dr_0 <= epp_data;
		when X"01" =>
			epp_dr_1 <= epp_data;
		when X"02" =>
			epp_dr_2 <= epp_data;
		when X"03" =>
			epp_dr_3 <= epp_data;
		when X"04" =>
			epp_dr_4 <= epp_data;
		when X"05" =>
			epp_dr_5 <= epp_data;
		when X"06" =>
			epp_dr_6 <= epp_data;
		when X"07" =>
			epp_dr_7 <= epp_data;
		when X"08" =>
			epp_dr_8 <= epp_data;
		when X"09" =>
			epp_dr_9 <= epp_data;
		when X"0a" =>
			epp_dr_10 <= epp_data;
		when X"0b" =>
			epp_dr_11 <= epp_data;
		when X"0c" =>
			epp_dr_12 <= epp_data;
		when X"0d" =>
			epp_dr_13 <= epp_data;
		when X"0e" =>
			epp_dr_14 <= epp_data;
		when X"0f" =>
			epp_dr_15 <= epp_data;
		when X"10" =>
			epp_dr_16 <= epp_data;
		when X"11" =>
			epp_dr_17 <= epp_data;
		when X"12" =>
			epp_dr_18 <= epp_data;
		when X"13" =>
			epp_dr_19 <= epp_data;
		when X"14" =>
			if epp_data = X"F0" then
				if br_idle = '1' then
					ending_host_data_write := false;
					br_fb_address <=  epp_dr_0 & epp_dr_1 & epp_dr_2 & epp_dr_3;
					br_start_strobe <= '1';
					epp_state	<= EPP_STARTING_BR;
					--we cannot set this yet. as it will take a cycle for the pr reader to become non-idle. set it it STARTING_BR
				end if; --otherwise we do a ending_host_data_write
			elsif epp_data = X"0F" then
				if bw_idle = '1' then
					ending_host_data_write := false;
					bw_fb_address <=  epp_dr_0 & epp_dr_1 & epp_dr_2 & epp_dr_3;
					bw_fb_data <= epp_dr_4 & epp_dr_5 & epp_dr_6 & epp_dr_7;
					bw_start_strobe <= '1';
					epp_state <= EPP_STARTING_BW;
				end if; --otherwise we do a ending_host_data_write
			elsif epp_data = X"03" then
				if bs_idle = '1' then
					ending_host_data_write := false;
					bs_fb_address <=  epp_dr_0 & epp_dr_1 & epp_dr_2 & epp_dr_3(7 downto 2);
					bs_num_bytes <= epp_dr_4 & epp_dr_5 & epp_dr_6(7 downto 2);
					bs_fb_value	<= epp_dr_7 & epp_dr_8 & epp_dr_9 & epp_dr_10;
					bs_start_strobe <= '1';
					epp_state <= EPP_STARTING_BS;
				end if;	 --otherwise we do a ending_host_data_write
			elsif epp_data = X"30" then
				if ps_idle = '1' then
					ending_host_data_write := false;
					ps_fb_address 	<= epp_dr_0 & epp_dr_1 & epp_dr_2 & epp_dr_3;
					ps_db_address 	<= epp_dr_4 & epp_dr_5 & epp_dr_6 & epp_dr_7;
					ps_start_z_depth <= epp_dr_8 & epp_dr_9 & epp_dr_10 & epp_dr_11;
					ps_z_increment	<= epp_dr_12 & epp_dr_13 & epp_dr_14 & epp_dr_15;
					ps_scanline_length <= epp_dr_16 & epp_dr_17;
					ps_pix_value	<= epp_dr_18;
					ps_start_strobe <= '1';
					epp_state <= EPP_STARTING_PS;
				end if;
			end if; --else we don't know what command was being sent, but we need to finish the cycle clealy so we do a ending_host_data_write. we could flag an error.
		when others =>
			--should never happen (invalid address in address register)
			--ending_host_data_write := true;
		end case;
		if ending_host_data_write = true then
			epp_wait <= '1';
			BeginSimpleDelay(EPP_ENDING_HOST_DATA_WRITE, X"014", epp_state, simple_delay_post_state, simple_delay_time, reset_wait_timer);
		end if;
	when EPP_DELAY_NOTIFY_HOST_DATA_READ=>
		epp_wait <= '1';
		BeginSimpleDelay(EPP_ENDING_HOST_DATA_READ, X"014", epp_state, simple_delay_post_state, simple_delay_time, reset_wait_timer);
	when EPP_STARTING_BR =>
		epp_data <= (others => 'Z');
		waiting_on_br <= '1';
		br_start_strobe <= '0';
		epp_wait <= '1';
		BeginSimpleDelay(EPP_ENDING_HOST_DATA_WRITE, X"014", epp_state, simple_delay_post_state, simple_delay_time, reset_wait_timer);
	when EPP_STARTING_BW =>
		epp_data <= (others => 'Z');
		bw_start_strobe <= '0';
		epp_wait <= '1';
		BeginSimpleDelay(EPP_ENDING_HOST_DATA_WRITE, X"014", epp_state, simple_delay_post_state, simple_delay_time, reset_wait_timer);
	when EPP_STARTING_BS =>
		epp_data <= (others => 'Z');
		bs_start_strobe <= '0';
		epp_wait <= '1';
		BeginSimpleDelay(EPP_ENDING_HOST_DATA_WRITE, X"014", epp_state, simple_delay_post_state, simple_delay_time, reset_wait_timer);
	when EPP_STARTING_PS =>
		epp_data <= (others => 'Z');
		ps_start_strobe <= '0';
		epp_wait <= '1';
		BeginSimpleDelay(EPP_ENDING_HOST_DATA_WRITE, X"014", epp_state, simple_delay_post_state, simple_delay_time, reset_wait_timer);
	when EPP_SIMPLE_DELAY =>
		if wait_timer = simple_delay_time then
			reset_wait_timer := true;
			epp_state <= simple_delay_post_state;
		end if;
	end case;
	
	if reset_wait_timer = true then
		wait_timer <= X"000";
	else
		wait_timer <= wait_timer + 1;
	end if;
	
  	end if; --reset
	end if; --clock
	end process;
end Behavioral;
