library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;


entity mcu_ll is
    Port (
		dram_d_buss		: inout std_logic_vector(31 downto 0);
		dram_a_buss 	: out std_logic_vector(11 downto 0);
		dram_ras_0 		: out std_logic;  -- all the rest of the dram signals are active low
		dram_ras_1 		: out std_logic;
		dram_ras_2 		: out std_logic;
		dram_ras_3 		: out std_logic;
		dram_cas_0 		: out std_logic;
		dram_cas_1 		: out std_logic;
		dram_cas_2 		: out std_logic;
		dram_cas_3 		: out std_logic;
		dram_we			: out std_logic;
		 
		tvc_clock		: in		std_logic;
		tvc_reset		: in 	std_logic;

		-- the io lines from the mcu_hl
		mcu_hl_data_in 			: in std_logic_vector(31 downto 0);
		mcu_hl_data_out			: out std_logic_vector(31 downto 0);
		mcu_hl_address				: in std_logic_vector(31 downto 2); --only read at startup
		mcu_hl_write_data			: in std_logic; 							--only read at startup
		mcu_hl_xfer_count			: in std_logic_vector(11 downto 0); --the number of transfers (not bytes) --only read at startup
		mcu_hl_data_valid_done 	: out std_logic; --either the mcu is done reading our data or has just written new data to us. high only a single cycle
		mcu_hl_start_strobe 		: in std_logic; --starts the entire transfer.
		mcu_hl_idle					: out std_logic --goes high when completly done with the transfer
		);
end mcu_ll;

architecture Behavioral of mcu_ll is

	signal dram_d_buss_out : std_logic_vector(31 downto 0);
	signal dram_d_buss_in  : std_logic_vector(31 downto 0);
	signal dram_we_sig : std_logic;
	
 	TYPE MCU_STATE is (
--	MCU_DUMMY_TEST,
	MCU_IDLE,
	MCU_DELAYED_XFER_REFRESH,
	MCU_IDLE_REFRESH,
	MCU_XFER_STARTUP_VALIDATE,
	MCU_SIMPLE_XFER,
	MCU_INT_XFER_REFRESH,
	MCU_ENDING_XFER,
	MCU_WAITSTATE );

	signal curr_state : MCU_STATE;
	signal after_waitstate_state : MCU_STATE;

	TYPE REFRESH_STATE is (
	REFRESH_DELAY_AFTER_ROW_ADDRESS,
	REFRESH_DELAY_AFTER_RAS_ASSERT,
	REFRESH_DELAY_AFTER_RAS_DEASSERT );
	signal mcu_refresh_state  : REFRESH_STATE;

	TYPE XFER_STATE is (
 	XFER_DELAY_AFTER_ROW_ADDRESS,
	XFER_DELAY_AFTER_RAS_ASSERT,
	XFER_DELAY_AFTER_COLUMN_ADDRESS,
	XFER_DELAY_AFTER_CAS_ASSERT,
	XFER_DELAY_AFTER_RAS_DEASSERT );
	signal dram_xfer_state  : XFER_STATE;

	signal cas_sig : STD_LOGIC;

	signal c_refresh_row : STD_LOGIC_VECTOR(11 downto 0);
	constant max_refresh_row : STD_LOGIC_VECTOR(11 downto 0) := X"3FF";

	--when this hits ?what?, time to refresh the next row.	 If the mcu is busy with another transfer, interrupt it
	--before the next CAS, do the refresh, then resume the transfer.	 it _could_ be the the same row as we are currently
	--in, but this is not very likly (don't check for eventuality) at a 50mhz clock we should have to refresh every 390 clocks.
	signal refresh_timer : STD_LOGIC_VECTOR(11 downto 0);
	signal refresh_needed : std_logic; 

	signal signal_timer : STD_LOGIC_VECTOR(7 downto 0); -- timing of all dram signals is handled by this timer.


	--this is the data 'parsed' from the selected mu's request registers.
	--we don't need to store all the values parsed. instead we directly use some of them.
	--the address in ram the mu is having us read/write from. I can only deal with the bottom ?21? bits of it with current hardware
	--signal mu_req_byte_address 	: STD_LOGIC_VECTOR(31 downto 0); (don't need to save this).
 	signal xfer_req_count	: STD_LOGIC_VECTOR(11 downto 0); --the number of transfers the mcu_hl is having us read/write for it.
	signal xfer_req_write	: std_logic; --if high we are writing.

	signal xfer_prev_dram_address_ras	: std_logic_vector(10 downto 0); --we use this to know if we must change RAS lines during a transfer
	signal xfer_curr_dram_address 		: std_logic_vector(31 downto 2); --set to what would have been in mu_req_byte_address
	signal xfer_end_dram_address  		: std_logic_vector(31 downto 2);
	signal xfer_curr_req_count				: std_logic_vector(11 downto 0); --this is the count/address of next r/w transfer that will occur

	procedure BeginRefreshCycle(
		signal mcu_refresh_state : out REFRESH_STATE;
		signal c_refresh_row 	: inout STD_LOGIC_VECTOR(11 downto 0);
		signal signal_timer 	: out STD_LOGIC_VECTOR(7 downto 0);
		signal dram_a_buss 		: out std_logic_vector(11 downto 0) ) is
	begin
		dram_a_buss <= c_refresh_row;
	 	mcu_refresh_state <= REFRESH_DELAY_AFTER_ROW_ADDRESS;
		signal_timer <= X"01";
		if c_refresh_row = max_refresh_row	 then
			c_refresh_row <= X"000";
		else
			c_refresh_row <= c_refresh_row + 1;
		end if;
	end BeginRefreshCycle;


	procedure InitializeXFER(
		signal curr_state					: out MCU_STATE;
		signal after_waitstate_state 	: out MCU_STATE;
		signal xfer_req_count			: out STD_LOGIC_VECTOR(11 downto 0);
		signal xfer_req_write			: out std_logic;
		signal xfer_curr_dram_address : out std_logic_vector(31 downto 2);
		signal xfer_end_dram_address  : out std_logic_vector(31 downto 2);
		signal xfer_curr_req_count		: out std_logic_vector(11 downto 0);
		signal mcu_hl_address			: in std_logic_vector(31 downto 2);
		signal mcu_hl_write_data		: in std_logic;
		signal mcu_hl_xfer_count		: in std_logic_vector(11 downto 0) ) is
	begin			
		xfer_req_count	<= mcu_hl_xfer_count;
		xfer_req_write	<= mcu_hl_write_data;
		xfer_curr_dram_address <= mcu_hl_address;
		xfer_end_dram_address <= mcu_hl_xfer_count + mcu_hl_address;
		xfer_curr_req_count	<= X"000";
		after_waitstate_state <= MCU_XFER_STARTUP_VALIDATE;
		curr_state <= MCU_WAITSTATE;
	end InitializeXFER;

 	--this is a seperate procedure because the functionality is needed in HandleRefresh and HandleXFERStartupValidate
	procedure XFERStart(
		signal curr_state					: inout MCU_STATE;
		signal dram_xfer_state 			: inout XFER_STATE;
		signal signal_timer 				: inout STD_LOGIC_VECTOR(7 downto 0);
		signal xfer_curr_dram_address : in std_logic_vector(31 downto 2);
		signal dram_a_buss 				: out std_logic_vector(11 downto 0) ) is
	begin
		dram_a_buss <= '0' & xfer_curr_dram_address(22 downto 12); --hack. check this value this might now be left shifted by two bits
		signal_timer <= X"01";  --one clock cycle will pass before this is examined again
		curr_state <= MCU_SIMPLE_XFER;
		dram_xfer_state <= XFER_DELAY_AFTER_ROW_ADDRESS;
	end XFERStart;

 	procedure HandleRefresh(
		signal curr_state					: inout MCU_STATE;
		signal after_waitstate_state 	: out MCU_STATE;
		signal mcu_refresh_state 		: inout REFRESH_STATE;
		signal dram_xfer_state 			: inout XFER_STATE;
		signal signal_timer 				: inout STD_LOGIC_VECTOR(7 downto 0);
		signal xfer_req_count			: out STD_LOGIC_VECTOR(11 downto 0);
		signal xfer_req_write			: out std_logic;
		signal xfer_curr_dram_address : inout std_logic_vector(31 downto 2);
		signal xfer_end_dram_address  : out std_logic_vector(31 downto 2);
		signal xfer_curr_req_count		: out std_logic_vector(11 downto 0);
		signal dram_a_buss 				: out std_logic_vector(11 downto 0);
		signal dram_ras_0 				: out std_logic;
		signal dram_ras_1 				: out std_logic;
		signal dram_ras_2 				: out std_logic;
		signal dram_ras_3 				: out std_logic;
		signal mcu_hl_address			: in std_logic_vector(31 downto 2);
		signal mcu_hl_write_data		: in std_logic;
		signal mcu_hl_xfer_count		: in std_logic_vector(11 downto 0);
		signal mcu_hl_start_strobe 	: in std_logic ) is
	variable state_change_needed : boolean := false;
	begin
		--curr state could be MCU_INT_XFER_REFRESH or MCU_DELAYED_XFER_REFRESH
		if curr_state = MCU_IDLE_REFRESH and mcu_hl_start_strobe = '1' then
			state_change_needed := true;
		end if;

		case mcu_refresh_state is
		when REFRESH_DELAY_AFTER_ROW_ADDRESS =>
			if state_change_needed = true then
				 curr_state <= MCU_DELAYED_XFER_REFRESH;
			end if;
			if signal_timer = x"01" then
				dram_ras_0 <= '0';
				dram_ras_1 <= '0';
				dram_ras_2 <= '0';
				dram_ras_3 <= '0';
				signal_timer <= X"01";
				mcu_refresh_state <= REFRESH_DELAY_AFTER_RAS_ASSERT;
			else
				signal_timer <= signal_timer + 1;
			end if;
		when REFRESH_DELAY_AFTER_RAS_ASSERT =>
			if state_change_needed = true then
				 curr_state <= MCU_DELAYED_XFER_REFRESH;
			end if;
			if signal_timer = x"04" then
				dram_ras_0 <= '1';
				dram_ras_1 <= '1';
				dram_ras_2 <= '1';
				dram_ras_3 <= '1';
				signal_timer <= X"01";
				mcu_refresh_state <= REFRESH_DELAY_AFTER_RAS_DEASSERT;
			else
				signal_timer <= signal_timer + 1;
			end if;
		when REFRESH_DELAY_AFTER_RAS_DEASSERT =>
			if signal_timer = x"02" then
				if curr_state = MCU_IDLE_REFRESH and state_change_needed = false then --we are done. otherwise we 
					curr_state <= MCU_IDLE; 
				elsif state_change_needed = true or curr_state = MCU_DELAYED_XFER_REFRESH then
					InitializeXFER(curr_state, after_waitstate_state, xfer_req_count, xfer_req_write, xfer_curr_dram_address, xfer_end_dram_address, xfer_curr_req_count,
										mcu_hl_address, mcu_hl_write_data, mcu_hl_xfer_count);
				else --resume an existing transfer 
					XFERStart( curr_state, dram_xfer_state, signal_timer,  xfer_curr_dram_address, dram_a_buss );
				end if;
			else
				if state_change_needed = true then
					curr_state <= MCU_DELAYED_XFER_REFRESH;
				end if;
				signal_timer <= signal_timer + 1;
			end if;
	 end case;
	end HandleRefresh;

	procedure HandleIdleClock(	signal curr_state					: inout MCU_STATE;
										signal after_waitstate_state 	: out MCU_STATE;
										signal mcu_refresh_state 		: out REFRESH_STATE;
										signal c_refresh_row 			: inout STD_LOGIC_VECTOR(11 downto 0);
										signal refresh_needed			: in std_logic;
										signal signal_timer 				: out STD_LOGIC_VECTOR(7 downto 0);
										signal dram_a_buss 				: out std_logic_vector(11 downto 0);
										signal mcu_hl_address			: in std_logic_vector(31 downto 2); --only read at startup
										signal mcu_hl_write_data		: in std_logic; --only read at startup
										signal mcu_hl_xfer_count		: in std_logic_vector(11 downto 0);
										signal xfer_req_count			: out STD_LOGIC_VECTOR(11 downto 0);
										signal xfer_req_write			: out std_logic;
										signal xfer_curr_dram_address : out std_logic_vector(31 downto 2);
										signal xfer_end_dram_address  : out std_logic_vector(31 downto 2);
										signal xfer_curr_req_count		: out std_logic_vector(11 downto 0) ) is
	begin
		if refresh_needed = '1' then 
			if mcu_hl_start_strobe = '1' then
				curr_state <= MCU_DELAYED_XFER_REFRESH;
			else
				curr_state <= MCU_IDLE_REFRESH;
			end if;
			BeginRefreshCycle(mcu_refresh_state, c_refresh_row, signal_timer, dram_a_buss);			
		else --see if the mcu_hl wants work
			if mcu_hl_start_strobe = '1' then
				InitializeXFER(curr_state, after_waitstate_state,
									xfer_req_count, xfer_req_write, xfer_curr_dram_address, xfer_end_dram_address, xfer_curr_req_count,
									mcu_hl_address, mcu_hl_write_data, mcu_hl_xfer_count);
			end if;
		end if;
	end HandleIdleClock;


 	procedure HandleXFERStartupValidate(
									signal curr_state					: inout MCU_STATE;
									signal after_waitstate_state 	: out MCU_STATE;
									signal dram_xfer_state 			: inout XFER_STATE;
									signal signal_timer 				: inout STD_LOGIC_VECTOR(7 downto 0);
									signal xfer_curr_dram_address : in std_logic_vector(31 downto 2);
									signal xfer_end_dram_address  : in std_logic_vector(31 downto 2);
									signal dram_a_buss 				: out std_logic_vector(11 downto 0) ) is
	begin
		if xfer_end_dram_address <= xfer_curr_dram_address then --there was an error. either the number of memory addesses was zero, or there was an overflow
			after_waitstate_state <= MCU_IDLE;
			curr_state <= MCU_WAITSTATE; --we need the mu to deassert its irq before we are actually idle.
		else
			--get ready to do much more work
			XFERStart(curr_state, dram_xfer_state, signal_timer, xfer_curr_dram_address, dram_a_buss);			
		end if;
	end HandleXFERStartupValidate;

	procedure HandleXFERData(
		signal curr_state							: inout MCU_STATE;
		signal dram_xfer_state 					: inout XFER_STATE;
		signal mcu_refresh_state 				: out REFRESH_STATE;
		signal c_refresh_row 					: inout STD_LOGIC_VECTOR(11 downto 0);
		signal refresh_needed					: in std_logic;
		signal signal_timer 						: inout STD_LOGIC_VECTOR(7 downto 0);
		signal xfer_prev_dram_address_ras 	: inout std_logic_vector(10 downto 0);
		signal xfer_curr_dram_address 		: inout std_logic_vector(31 downto 2);
		signal xfer_end_dram_address  		: in std_logic_vector(31 downto 2);
		signal dram_d_buss_in					: in std_logic_vector(31 downto 0);
		signal dram_d_buss_out					: out std_logic_vector(31 downto 0);
		signal dram_a_buss 						: out std_logic_vector(11 downto 0);
		signal dram_ras_0 						: out std_logic;
		signal dram_ras_1 						: out std_logic;
		signal dram_ras_2 						: out std_logic;
		signal dram_ras_3 						: out std_logic;
		signal cas_sig 							: out std_logic;
		signal dram_we_sig						: out std_logic;
		signal mcu_hl_data_in					: in std_logic_vector(31 downto 0);
		signal mcu_hl_data_out					: out std_logic_vector(31 downto 0);
		signal mcu_hl_data_valid_done 		: out std_logic ) is
	begin
	case dram_xfer_state is
		when XFER_DELAY_AFTER_ROW_ADDRESS =>
			if signal_timer = x"02" then
				if xfer_curr_dram_address(22) = '0' then
					dram_ras_0 <= '0';
					dram_ras_2 <= '0';
					dram_ras_1 <= '1'; --should be no change
					dram_ras_3 <= '1';	--no change
				else
					dram_ras_0 <= '1'; --no change
					dram_ras_2 <= '1'; --no change
					dram_ras_1 <= '0';
					dram_ras_2 <= '0';
				end if;
				signal_timer <= X"01";
				dram_xfer_state <= XFER_DELAY_AFTER_RAS_ASSERT;
			else
				signal_timer <= signal_timer + 1;
			end if;
		when XFER_DELAY_AFTER_RAS_ASSERT =>
			if signal_timer = x"02" then
				dram_a_buss <= "00" & xfer_curr_dram_address(11 downto 2);
				signal_timer <= X"01";
				dram_xfer_state <= XFER_DELAY_AFTER_COLUMN_ADDRESS;
				if xfer_req_write = '1' then	
					dram_we_sig <= '0';	--I don't know if I should do this here or when I assert the CAS
					dram_d_buss_out <= mcu_hl_data_in;	--get the data presented to the dram so when we assert the cas the dram will see it.
					mcu_hl_data_valid_done <= '1';
				end if;
			else
				signal_timer <= signal_timer + 1;
			end if;
		when XFER_DELAY_AFTER_COLUMN_ADDRESS=>
			mcu_hl_data_valid_done <= '0'; --only necessary when we do a fpm cycle
			if signal_timer = x"02" then
				cas_sig <= '0';
				signal_timer <= X"01";
				dram_xfer_state <= XFER_DELAY_AFTER_CAS_ASSERT;
				--compute a the next dram_address. we need this done now so we can use the results in the next state (to know if we should end the xfer
				xfer_prev_dram_address_ras <= xfer_curr_dram_address(22 downto 12);
				xfer_curr_dram_address <= xfer_curr_dram_address + 1;
			else
				signal_timer <= signal_timer + 1;
			end if;
		when XFER_DELAY_AFTER_CAS_ASSERT=>
 			if signal_timer = x"02" then	--if reading then data should be availiable
				if xfer_req_write = '0' then
					mcu_hl_data_out <= dram_d_buss_in;
					--mu_buss_data_out <= X"12345678";
					--case xfer_curr_dram_address(3 downto 2) is
					--when "00" => mu_buss_data <= X"C0C0C0C0";
					--when "01" => mu_buss_data <= X"38383838";
					--when "10" => mu_buss_data <= X"07070707";
					--when "11" => mu_buss_data <=  X"00000000";
					--when others => mu_buss_data <= X"00000000";
					--end case;
					mcu_hl_data_valid_done <= '1';
				end if;
				if xfer_curr_dram_address = xfer_end_dram_address or
				   xfer_prev_dram_address_ras /= xfer_curr_dram_address(22 downto 12) or
				   refresh_needed = '1'
				then --our transfer is 'done' or we must change our RAS
					signal_timer <= X"01";
					dram_xfer_state <= XFER_DELAY_AFTER_RAS_DEASSERT;
					dram_ras_0 <= '1';
					dram_ras_1 <= '1';
					dram_ras_2 <= '1';
					dram_ras_3 <= '1';
					cas_sig <= '1';
					dram_we_sig <= '1'; --we will reset it at our next transfer
				else --we are not done with our transfer _and_ we are staying in the current RAS line. compute and present a new CAS
					if xfer_req_write = '1' then
						dram_d_buss_out <= mcu_hl_data_in; --present next data to simm.
						mcu_hl_data_valid_done <= '1';
						--dram_we_sig <= '1'; if we were writing, we will keep writing, don't change this
					end if; --if reading we handled it above
					dram_a_buss <= "00" & xfer_curr_dram_address(11 downto 2);
					signal_timer <= X"01";
					dram_xfer_state <= XFER_DELAY_AFTER_COLUMN_ADDRESS;
					cas_sig <= '1';
				end if;
			else
				signal_timer <= signal_timer + 1;
			end if;
		when XFER_DELAY_AFTER_RAS_DEASSERT =>
			mcu_hl_data_valid_done <= '0';
			if signal_timer = x"02" then --we _must_ delay for two cycles.
				if xfer_curr_dram_address = xfer_end_dram_address then --the transfer is done 
					curr_state <= MCU_ENDING_XFER;
				elsif refresh_needed = '1' then
					 curr_state <= MCU_INT_XFER_REFRESH;
					BeginRefreshCycle(mcu_refresh_state, c_refresh_row, signal_timer, dram_a_buss);
				else --changing RAS lines
					dram_a_buss <= xfer_curr_dram_address(23 downto 12);
					signal_timer <= X"00";
					dram_xfer_state <= XFER_DELAY_AFTER_ROW_ADDRESS;
				end if;
			else
				signal_timer <= signal_timer + 1;
			end if;
	 end case;
	 end HandleXFERData;

begin

	dram_d_buss_in <= dram_d_buss;
	dram_we <= dram_we_sig;

	process(dram_we_sig, dram_d_buss_out) is
	begin
		if dram_we_sig = '0' then
			dram_d_buss <= dram_d_buss_out;
		else
			dram_d_buss <= (others => 'Z');
		end if;
	end process;
	
	process(cas_sig) is
	begin
		dram_cas_0 <= cas_sig;
          dram_cas_1 <= cas_sig;
          dram_cas_2 <= cas_sig;
          dram_cas_3 <= cas_sig;
	end process;

	process(curr_state) is
	begin
		if curr_state = MCU_IDLE OR curr_state = MCU_IDLE_REFRESH then
			mcu_hl_idle <= '1';
		else
			mcu_hl_idle <= '0';
		end if;
	end process;

	process(tvc_clock)
	begin
	if tvc_clock = '1' and tvc_clock'event then
	if tvc_reset = '1' then
		dram_a_buss <= X"000";
		dram_ras_0 <= '1';
		dram_ras_1 <= '1';
		dram_ras_2 <= '1';
		dram_ras_3 <= '1';
      cas_sig <= '1';

 		dram_we_sig <= '1';
		dram_d_buss_out <= (others => '0');

		mcu_hl_data_out <= X"0000_0000";
		mcu_hl_data_valid_done 	<= '0';
		
		--curr_state <= MCU_DUMMY_TEST;
		curr_state <= MCU_IDLE;
		mcu_refresh_state <= REFRESH_DELAY_AFTER_ROW_ADDRESS;
		dram_xfer_state <= XFER_DELAY_AFTER_ROW_ADDRESS;

		xfer_req_count <= X"000";
	   xfer_req_write <= '0';

 		c_refresh_row <= X"000";
		refresh_needed <= '0';
		refresh_timer <= X"000";
		--refresh_timer <= X"23A";
		c_refresh_row <= X"005";

		signal_timer <= X"00";
	else	--not a reset
	  	--set to 760. could be 781! we trigger early because it could be several cycles before we can actually start the refresh.
		--have not counted actual cycles yet
		if refresh_timer = X"2F8" then
			if curr_state /= MCU_INT_XFER_REFRESH and curr_state /= MCU_IDLE_REFRESH then
				refresh_needed <= '1';
			end if;
			refresh_timer <= X"000";
		else
			refresh_timer <= refresh_timer + 1;
		end if;

		case curr_state is
--		when MCU_DUMMY_TEST =>
--			if signal_timer = X"00" then
--				mcu_hl_data_out <= X"F0F0F0F0";
--				signal_timer <= signal_timer + 1;
--			elsif signal_timer = X"01" then		
--				mcu_hl_data_out <= X"0F0F0F0F";
--				signal_timer <= signal_timer + 1;
--			elsif signal_timer = X"02" then
--				mcu_hl_data_out <= X"aFbFcFdF";
--				signal_timer <= signal_timer + 1;
--			else
--				dram_d_buss_out <= mcu_hl_data_in;
--				signal_timer <= X"00";
--			end if;
		when MCU_IDLE =>
			HandleIdleClock(curr_state, after_waitstate_state, mcu_refresh_state, c_refresh_row, refresh_needed, signal_timer,
								dram_a_buss, mcu_hl_address, mcu_hl_write_data, mcu_hl_xfer_count, xfer_req_count, xfer_req_write, xfer_curr_dram_address,
								xfer_end_dram_address, xfer_curr_req_count);
		when MCU_IDLE_REFRESH | MCU_DELAYED_XFER_REFRESH | MCU_INT_XFER_REFRESH =>
			refresh_needed <= '0';
			HandleRefresh(curr_state, after_waitstate_state, mcu_refresh_state, dram_xfer_state, signal_timer,
				xfer_req_count, xfer_req_write, xfer_curr_dram_address, xfer_end_dram_address, xfer_curr_req_count,
				dram_a_buss, dram_ras_0, dram_ras_1, dram_ras_2, dram_ras_3,
				mcu_hl_address, mcu_hl_write_data, mcu_hl_xfer_count, mcu_hl_start_strobe);
		when MCU_XFER_STARTUP_VALIDATE =>
			HandleXFERStartupValidate(curr_state, after_waitstate_state, dram_xfer_state, signal_timer,
												xfer_curr_dram_address, xfer_end_dram_address, dram_a_buss);
		when MCU_SIMPLE_XFER=>
			HandleXFERData(curr_state, dram_xfer_state, mcu_refresh_state,
						c_refresh_row, refresh_needed, signal_timer,
						xfer_prev_dram_address_ras, xfer_curr_dram_address, xfer_end_dram_address,
 						dram_d_buss_in, dram_d_buss_out, dram_a_buss, dram_ras_0, dram_ras_1, dram_ras_2, dram_ras_3, cas_sig, dram_we_sig,
						mcu_hl_data_in, mcu_hl_data_out, mcu_hl_data_valid_done);
		when MCU_ENDING_XFER=>
			after_waitstate_state <= MCU_IDLE;
			curr_state <= MCU_WAITSTATE;
		when MCU_WAITSTATE=>
			curr_state <= after_waitstate_state;
		end case;
	end if; --reset
	end if; --clock
	end process;
	
end Behavioral;
